/*-
 * Copyright (c) 2020 UPLEX Nils Goroll Systemoptimierung
 * All rights reserved
 *
 * Author: Geoffrey Simmons <geoffrey.simmons@uplex.de>
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 * 1. Redistributions of source code must retain the above copyright
 *    notice, this list of conditions and the following disclaimer.
 * 2. Redistributions in binary form must reproduce the above copyright
 *    notice, this list of conditions and the following disclaimer in the
 *    documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE AUTHOR AND CONTRIBUTORS ``AS IS'' AND
 * ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
 * ARE DISCLAIMED.  IN NO EVENT SHALL AUTHOR OR CONTRIBUTORS BE LIABLE
 * FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL
 * DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS
 * OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 * HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT
 * LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY
 * OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
 * SUCH DAMAGE.
 */

#include <errno.h>
#include <limits.h>
#include <unistd.h>

#include "vsb.h"

struct qp_y;

struct match_data {
	unsigned	magic;
#define MATCH_DATA_MAGIC 0x0d9a845e
	unsigned	*indices;
	unsigned	limit;
	unsigned	n;
	unsigned	exact;
	unsigned	min;
	unsigned	max;
};

struct qp_stats {
	unsigned	magic;
#define QP_STATS_MAGIC 0x06d2b30c
	uint64_t	nodes;
	uint64_t	leaves;
	uint64_t	terms;
	uint64_t	nodesz;
	uint64_t	dmin;
	uint64_t	dmax;
	double		davg;
	uint64_t	fmin;
	uint64_t	fmax;
	double		favg;
};

int QP_Insert(struct qp_y * * restrict root, unsigned idx,
	      char * const restrict * const restrict strings,
	      unsigned allow_overlaps);
unsigned QP_Lookup(const struct qp_y * const restrict root,
		   char * const restrict * const restrict strings,
		   const char * const restrict subject);
int QP_Prefixes(const struct qp_y * const restrict root,
		char * const restrict * const restrict strings,
		const char * const restrict subject,
		struct match_data * const restrict match);
void QP_Stats(const struct qp_y * const restrict root,
	      char * const restrict * const restrict strings,
	      struct qp_stats * const restrict stats);
void QP_Free(struct qp_y *y);
struct vsb * QP_Dump(struct qp_y *root, char **strings);
